// Data history viewer for SensorNet  (HVAC status + indoor temp) and online weather merged
// Left-click = zoom in,   Right-click = zoom out,  Left+move = drag

Pm.Echo() // clear console

outside = 1			// outdoor range
Pm.Window( 'HVAC' )
ReadLogs()		// date, temp, rh
moved = 0
pos = 0
gas = false
dateNow = new Date()
dateStart = dateNow  // start date of mouse down

threshold = 1.7

	kwh = 3600 // killowatt hours (compressor+fan)
	ppkwh = 0.152  // price per KWH  (price / KWH)
	ppkwh  += 0.101 + 0.03 // surcharge 10.1%, school 3%, misc

	ccfs = 0.70 / (60*60)			// 0.70 cost per cubic feet per hour / into seconds

Setup()
Draw(0)

Pm.SetTimer(4*60*1000)

function Setup()
{
	h1 = ArrayDuration( listOut ) // max hours of data
	h2 = ArrayDuration( listIn )
	if(h1 < h2) h1 = h2
	h1++		// max hours of data
	h_show = 8 * 24	// max hours to show
	if(h_show > h1 ) h_show = h1

	// Temperature range

	min = arrayMin(listOut, 1, 200)
	max = arrayMax(listOut, 1, -200)

	min = arrayMin(listIn, 1, min)
	max = arrayMax(listIn, 1, max)
	inmin = min
	inmax = max

	if(listHvac != undefined)
	{
		min = arrayMin(listHvac, 5, min)
		max = arrayMax(listHvac, 5, max)
	}

	if(listStat != undefined && listStat.length)
	{
		min = arrayMin(listStat, 3, min)
		max = arrayMax(listStat, 3, max)
	}
}

function OnMouse(Msg, x, y)
{
	switch(Msg)
	{
		case 'LDN':		// left button down
			moved = 0
			pos = x
			if(x < left)
			{
				ReadLogs()
			}else{
				dateStart = dateNow
			}
			break
		case 'LUP':
			break
		case 'MOVE':	// drag to slide
			inc = (right - left) / h_show
			dt = new Date()
			if( (pos-x) == 0) break
			dateNow = dateStart - ((6000*((right-left) / inc)) * (-(pos-x) ) )
			moved = 1
			break
		case 'WHEEL':		// wheel zoom
			m = (h_show / 10)
			if(m < 1) m = 1
			h_show -= x * m
			h_show = Math.floor(h_show)
			if(h_show < 8) h_show = 8
			if(h_show > 182) h_show = 182
			break
	}
	Draw(0)
}

function OnCall(msg, a1)
{
	switch(msg)
	{
		case 'REFRESH':
				ReadLogs()
				Draw(0)
				break
	}
}

function OnTimer()
{
	ReadLogs()
	Draw(1)
	Pm.SetTimer(4*60*1000)
}

// Let's draw this
function Draw(fast)
{
	if(!fast)
	{
		Gdi.Width = 736 // resize drawing area/window
		Gdi.Height = 360

		left = 60 // chart area
		top = 23
		bottom = Gdi.Height - 26
		right = Gdi.Width - 34
		Gdi.Clear(0) // all trasparent

		Gdi.Brush( Gdi.Argb(120, 0, 0, 240) ) // drag bar
		Gdi.FillRectangle(0, 0, Gdi.Width-1, 26, 6)

		// rounded window
		Gdi.Clear(0, 1, 16, Gdi.Width-2, 16) // erase bottom of drag bar
		Gdi.Brush( Gdi.Argb(100, 0,0,0) ) // background
		Gdi.FillRectangle(0, 0, Gdi.Width-1, Gdi.Height-1, 6)
		Gdi.Pen( Gdi.Argb(120, 0, 0, 240), 1 ) // blue borders
		Gdi.Rectangle(0, 0, Gdi.Width-1, Gdi.Height-1, 6)

		// Title
		Gdi.Font( 'Courier New', 14, 'BoldItalic')
		Gdi.Brush( Gdi.Argb(255, 255, 230, 25) )
		Gdi.Text( 'HVAC History', 10, 0 )

		Gdi.Brush(Gdi.Argb(255, 255, 0, 0) ) // close button
		Gdi.Text('X', Gdi.Width - 18, 1)
	}
	// chart BG
	Gdi.Brush( Gdi.Argb(235, 0, 0, 0) )
	Gdi.FillRectangle( left-1, top, right-left, bottom-top )
	Gdi.Font( 'Courier New', 10)
	Gdi.Brush( Gdi.Argb(255, 255, 255, 250) )  // text color
	Gdi.Pen( Gdi.Argb(255, 50, 50, 50), 1 )   // grid color

		// Temp scale and H-lines
		y = top - 6
		incy = (bottom-top)/9
		dec = (max-min)/9
		t = max
		for(i = 0; i <= 9; i++)
		{
			x = right+20-( (t<0) ? 25:19 ) // shift negative left
			if( t > -10 && t < 10) x += 6 // single digit
			Gdi.Text( t.toFixed(0), x, y )
			Gdi.Line(left, y+6, right+2, y+6)
			y += incy
			t -= dec
		}

	inc = (right - left) / (h_show * 60)

	Gdi.Font( 'Courier New', 10) // time totals
	Gdi.Pen(Gdi.Argb(255, 0, 0, 0), 1)

// Sparkostat logs
	xa = 0
	tot = 0
	totalCost = 0
	dayCost = 0
	totalTime = 0

	if( listHvac.length )
	{
		ptTemp = new Array()
		ptRh = new Array()
		l = h = xl = yl = xh = yh = 0 // low - high and positions

			ldate = new Date(listHvac[listHvac.length-1][0] * 1000)
			d2 = new Date(ldate.getYear(), ldate.getMonth(), ldate.getDate())
			ptx = (dateNow - d2) / (1000 * 60)
			xa = (right + 3 - (ptx * inc)) + (inc * 1400) // default 1 day fwd for partial

		for (i = listHvac.length-1; i >= 0; i--)
		{
			date = new Date( listHvac[i][0] * 1000)
			pt = (dateNow - date) / (1000 * 60)
			if(date.getDate() != ldate.getDate())	// day change
			{
				Gdi.Brush(Gdi.Argb(255, 255, 255, 255) )
				d2 = new Date(ldate.getYear(), ldate.getMonth(), ldate.getDate())
				ptx = (dateNow - d2) / (1000 * 60)
				xa = right + 3 - (ptx * inc)
				if(dayCost){
					Gdi.OutlineText(Pm.TimeStr(tot), xa, bottom, 3)
					Gdi.OutlineText('$' + dayCost.toFixed(2), xa, bottom+12, 3)
				}
				dayCost = 0
				tot = 0
				ldate = date
			}

			x = right - (pt * inc)
			s = +listHvac[i][2]
			date2 = new Date(date.getTime() - (1000*s))

			totalTime += s // seconds
			tot += s

					if(+listHvac[i][1] != 3)
						cost = ppkwh * s / (1000*60*60) * kwh
					else
					{
						gas = true
						cost = ccfs * s
					}
					totalCost += cost
					dayCost += cost

			pt2 = (dateNow - date2) / (1000 * 60)
			x2 = right - (pt2 * inc)

			t = +listHvac[i][5]
			y = bottom - (t - min) * (bottom-top-1) / (max-min)
			ptTemp[ptTemp.length] = x
			ptTemp[ptTemp.length] = y
			if( h <= t){ h = t; xh = x; yh = y} // highest
			if( l >= t){ l = t; xl = x; yl = y} // lowest

			t = +listHvac[i][3]
			y = bottom - (t - min) * (bottom-top-1) / (max-min)
			ptTemp[ptTemp.length] = x2
			ptTemp[ptTemp.length] = y
			if( h <= t){ h = t; xh = x2; yh = y} // highest
			if( l >= t){ l = t; xl = x2; yl = y} // lowest

			r = (+listHvac[i][6]) * (bottom-top) / 100
			ptRh[ptRh.length] = x
			ptRh[ptRh.length] = bottom - r

			r = (+listHvac[i][4]) * (bottom-top) / 100
			ptRh[ptRh.length] = x2
			ptRh[ptRh.length] = bottom - r

			hour = date.getHours()
			if(hour == 0) // midnight to prev noon
			{
				if(l && xh){
					Gdi.Brush(Gdi.Argb(255, 255, 255, 255))
					Gdi.Text(l.toFixed(1), xl+((yl < top+7) ? 3:-8), ((yl < top+7) ? top-1:yl-9)) // upper temps
				}
				h = t
			}
			if(hour == 12) // noon to midnight
			{
				if(xl )
				{
					Gdi.Brush(Gdi.Argb(255, 255, 255, 255))
					Gdi.Text(h.toFixed(1), xh+((yh+7 > bottom) ? 3:-8),  ((yh+7 > bottom) ? bottom-8:yh-2)) // lower temps
				}
				l = t
			}

			if( x < left) continue;

			switch(+listHvac[i][1])
			{
				case 1:	// cool
						Gdi.Brush(Gdi.Argb(50, 20, 20, 255))
						break
				case 2: 	// heat
				case 3: 	// ng
						Gdi.Brush(Gdi.Argb(30, 155, 70, 70))
						break
			}

			if( x< right)
			{ // Sparkostat runs
				Gdi.FillRectangle( x2, 238, x-x2, 72 )
			}
		}

		if(tot)
		{
			Gdi.Brush(Gdi.Argb(255, 255, 255, 255) )
			xa -= inc * 1400
			if(xa > left){
				Gdi.OutlineText( Pm.TimeStr(tot), xa, bottom, 3)
				Gdi.OutlineText('$' + dayCost.toFixed(2), xa, bottom+12, 3)
			}
		}

		y = (max-min) * (bottom - top) / (max-min)
		// log temps color
	//	Gdi.GradientBrush(left, top, Gdi.Width, 100+y, Gdi.Argb(100, 120,0x0, 200), Gdi.Argb(100, 128,0,255), 90)
	//	Gdi.Lines( ptTemp )

	//	Gdi.Pen( Gdi.Argb(20, 0, 230, 230), 1 )
	//	Gdi.Lines( ptRh )
	}

// Sparkostat query
	ptTemp = new Array()
	ptTemp2 = new Array()
	ptTemp3 = new Array()
	ptRh = new Array()

	if( listStat.length )
	{
		for (i = listStat.length-1; i >= 0 && x >= left; i --)
		{
			date = new Date(listStat[i][0] * 1000)

			pt = (dateNow - date) / (1000 * 60)
			x = right - (pt * inc)
			if(x > right) continue

			t = +listStat[i][3]	// temp
			y = bottom - (t - min) * (bottom-top-1) / (max-min)

			ptTemp[ptTemp.length] = x
			ptTemp[ptTemp.length] = y

			t = +listStat[i][4] // target
			y = bottom - (t - min) * (bottom-top-1) / (max-min)
			ptTemp2[ptTemp2.length] = x
			ptTemp2[ptTemp2.length] = y

			if(+listStat[i][1] == 1) t -= threshold // cool
			else t += threshold // heat
			y = bottom - (t - min) * (bottom-top-1) / (max-min)
			ptTemp3[ptTemp3.length] = x
			ptTemp3[ptTemp3.length] = y

			y = (+listStat[i][5]) * (bottom-top) / 100 // in RH
			ptRh[ptRh.length] = x
			ptRh[ptRh.length] = bottom - y
		}

		y = (max-min) * (bottom - top) / (max-min)
		// inTemp color
		Gdi.Pen(Gdi.Argb(160, 255, 255, 55), 1)
		Gdi.Lines( ptTemp )

		// TargetTemp color
		Gdi.Pen(Gdi.Argb(180, 80, 80, 255), 1)
		Gdi.Lines( ptTemp2 )
		Gdi.Pen(Gdi.Argb(185, 80, 80, 255), 1)
		Gdi.Lines( ptTemp3 )

		Gdi.Pen(Gdi.Argb(35, 0, 255, 255), 1)
		Gdi.Lines( ptRh )
	}

	if(!fast)
	{
		Gdi.Font( 'Courier New', 10, 'Bold')
		y = 22
		Gdi.Pen(Gdi.Argb(255, 0, 0, 0), 1 )
		Gdi.Brush( Gdi.Argb(255, 255, 255, 255) )
		Gdi.OutlineText('Spark', 2, y+=11, 3)
		Gdi.Brush( Gdi.Argb(255, 100, 100, 255) )
		Gdi.OutlineText('Target', 2, y+=11, 3)

		Gdi.Brush( Gdi.Argb(255, 220, 10, 255) )
		Gdi.Brush( Gdi.Argb(255, 255, 255, 55) )
		Gdi.OutlineText('Query', 2, y+=11, 3 )
		Gdi.Brush( Gdi.Argb(255, 0, 240, 240) )
		Gdi.OutlineText('rH%', 2, y+=11, 3 )

	}

	Gdi.Font( 'Courier New', 10)
	// Days and grid
	days = Array('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat')
	date1 = new Date(dateNow)
	day = date1.getDay()
	d = date1.getDate()
	hour = date1.getHours()
	// Grid
	Gdi.Brush( Gdi.Argb(255, 255, 255, 0) )
	if(listOut.length)
	{
		date = new Date( listOut[listOut.length-1][0] * 1000)
		date1.setMinutes(date.getMinutes())
		pt = (dateNow.valueOf() - date1.valueOf()) / (1000 * 60)
		x = right - (pt * inc)

		for(i = 1000; i >= 0; i--)  // v-lines
		{
			if(x < left) break

			if(hour == 0)
			{
				Gdi.Pen( Gdi.Argb(200, 80, 80, 80), 1 )   // midnight
				Gdi.Line(x, top, x, bottom)
				Gdi.Pen( Gdi.Argb(200, 0, 0, 1), 1 )
				if(d < 1 ) d = getDays(date.getMonth(), date.getYear()) // end of month wrap
				if( x < right) Gdi.OutlineText(d + ' ' + days[day], x + 60, bottom + 1, 5)
				if(--day <0) day = 6
				d--
			}
			if(hour == 12)
			{
				Gdi.Pen( Gdi.Argb(100, 50, 50, 50), 1 )   // noon
				Gdi.Line(x, top, x, bottom)
			}

			h = hour
			if(h > 12) h -= 12		// 12h format
			if(h == 0) h = 12
			if(inc < 0.06 && (hour &7));	// hour skip if too close
			else if(inc < 0.14 && (hour &3));	// hour skip if too close
			else if(inc < 0.24 && (hour &1));
			else Gdi.Text( h, x, top-9, 'Center' )

			if(h_show < 48)	// hourly lines
			{
				Gdi.Pen( Gdi.Argb(100, 50, 50, 50), 1 )
				Gdi.Line(x, top, x, bottom)
			}

			if(--hour < 0) hour = 23
			x -= (inc * 60)
		}
	}
	// Now arrow
	date0 = new Date()
	Gdi.Pen( Gdi.Argb(255, 255, 0, 0), 1 )
	pt = (dateNow - date0) / (1000 * 60)
	x = right - (pt * inc)
	if(x >= left && x <= right)
	{
		Gdi.Line(x, bottom, x+3, bottom+6)
		Gdi.Line(x, bottom, x-3, bottom+6)
		Gdi.Line(x, top, x+3, top-6)
		Gdi.Line(x, top, x-3, top-6)
	}

// outdoor Temp/rH
	if(listOut.length)
	{
		Gdi.Font( 'Courier New', 10) // temp peaks font
		Gdi.Brush( Gdi.Argb(255, 255, 230, 250) )  // text color

		tempxy = Array()
		rh = Array()
		l = h = x3 = y3 = x2 = y2 = 0 // low - high and positions

		for (i = listOut.length-1, n = 0, x = right; i >= 0; i--)
		{
			t = +listOut[i][1]
			hu = (+listOut[i][2]) * (bottom-top) / 100
			y = bottom - (t - min) * (bottom-top-1) / (max-min)

			date = new Date( listOut[i][0] * 1000)
			pt = (dateNow - date) / (1000 * 60)
			x = right - (pt * inc)
			hour = date.getHours()

			if(x > right) continue

			tempxy[n] = x // position of temp
			rh[n++] = x
			tempxy[n] = y
			rh[n++] = bottom - hu
			if(x < left) 	break

			if( h <= t){ h = t; x2 = x; y2 = y} // highest
			if( l >= t){ l = t; x3 = x; y3 = y} // lowest
			if(hour == 0) // midnight to prev noon
			{
				if(x3) Gdi.Text(l, x3+((y3+7 > bottom) ? 3:-8),  ((y3+7 > bottom) ? bottom-8:y3-2)) // lower temps
				h = t
			}
			if(hour == 12) // noon to midnight
			{
				if(x2) Gdi.Text(h, x2+((y2 < top+7) ? 3:-8), ((y2 < top+7) ? top-1:y2-9)) // upper temps
				l = t
			}
		}

		if(min < 32) // make line blue below freezing
		{
			y = (32 - min) * (bottom - top) / (max-min)
			Gdi.GradientBrush(left, bottom - y , Gdi.Width, 1000,
			Gdi.Argb( 255, 255, 0, 0), Gdi.Argb( 255, 0, 0, 255), -90)
		}
		else // normal color
		{
			y = (max-min) * (bottom - top) / (max-min)
			Gdi.GradientBrush(left, top, Gdi.Width, 100+y, 0xFFFF0000, 0xFF0000FF, 90)
		}
		Gdi.Lines( tempxy )

		// out rH color
		Gdi.Pen( Gdi.Argb(10, 0,200, 190), 1 )
		Gdi.Lines( rh )
	}

// Indoor temp/rH history
	if(listIn.length)
	{
		Gdi.Font( 'Courier New', 10) // temp peaks font
		Gdi.Brush( Gdi.Argb(255, 255, 180, 20) )  // text color
		tempxy = Array()
		rh = Array()
		l = h = x3 = y3 = x2 = y2 = 0 // low - high and positions
		tog = 0
		for (i = listIn.length-1, n = 0, x = right; i >= 0; i--, x -= inc)
		{
			t = +listIn[i][1]
			t = t.toFixed(1)
			hu = (+listIn[i][2]) * (bottom-top) / 100

			y = bottom - (t - min) * (bottom-top) / (max-min)

			date = new Date( listIn[i][0] * 1000 )
			hour = date.getHours()
			pt = (dateNow - date) / (1000 * 60)
			x = right - (pt * inc)

			if(x > right) continue

			tempxy[n] = x // position of temp
			rh[n++] = x
			tempxy[n] = y
			rh[n++] = bottom - hu

			if(x < left) break

			if( h <= t){ h = t; x2 = x; y2 = y} // highest
			if( l >= t){ l = t; x3 = x; y3 = y} // lowest
			if(hour == 12 && tog != 2) // midnight to noon
			{
				tog = 2
				Gdi.Text(h, x2+((y2 < top+7) ? 3:-8), ((y2 < top+7) ? top-1:y2-9)) // upper temps
				l = t
			}
			if(hour == 0 && tog != 3) // noon to midnight
			{
				tog = 3
				Gdi.Text(l, x3+((y3+7 > bottom) ? 3:-8),  ((y3+7 > bottom) ? bottom-8:y3-2)) // lower temps
				h = t
			}
		}
		y = (max-min) * (bottom - top) / (max-min)
		// in temp color
		Gdi.GradientBrush(left, top, Gdi.Width, 100+y, 0x7FFF8000, 0x7F800000, 90)

		Gdi.Lines( tempxy )
		Gdi.Font( 'Courier New', 11, 'BoldItalic')
		t = listIn[listIn.length-1][1]
		y = bottom - ((t-min) * (bottom - top) / (max-min))
		Gdi.Brush( Gdi.Argb(255, 255,210, 0) )
		Gdi.Pen( Gdi.Argb(100, 0,0, 0), 1 )
		Gdi.OutlineText(t, right, y-6, 5)

		// in rH color
		Gdi.Pen( Gdi.Argb(20, 0, 255, 100), 1 )
		Gdi.Lines( rh )
		t = listIn[listIn.length-1][2]
		y = bottom - (t * (bottom - top) / 100)
		Gdi.Pen( Gdi.Argb(100, 0,0, 0), 1 )
		Gdi.Brush( Gdi.Argb(100, 100, 255, 200) )
		Gdi.OutlineText(t, right, y-6, 4, 'Right')
	}

	// Legend
	if(!fast)
	{
		y = 90
		Gdi.Pen(Gdi.Argb(255, 0, 0, 0), 1 )
		Gdi.Brush( Gdi.Argb(255, 200, 200, 200) )
		Gdi.OutlineText( 'Out', 2, y+=11, 3)
		Gdi.Brush( Gdi.Argb(255, 255, 60, 60) )
		Gdi.OutlineText( min+'-'+max+'', 2, y+=11, 3)
		Gdi.Brush( Gdi.Argb(200, 0,160, 160) )
		Gdi.OutlineText( 'rH%', 2, y+=11, 3)

		Gdi.Brush( Gdi.Argb(255, 200, 200, 200) )
		Gdi.OutlineText( 'Sensor2', 2, y+=20, 3)
		Gdi.Brush( Gdi.Argb(255, 240, 168, 80) )
		Gdi.OutlineText( inmax.toFixed(1)+'', 2, y+=11, 3)
		Gdi.OutlineText( inmin.toFixed(1)+'', 2, y+=11, 3)
		Gdi.Brush( Gdi.Argb(255, 0,255, 100) )
		Gdi.OutlineText( 'rH%', 2, y+=11, 3)

		Gdi.Brush( Gdi.Argb(255, 80,80, 255) )
		Gdi.OutlineText( 'Cycles', 2, y+=55, 3)

		// Run
		y = Gdi.Height - 100
		Gdi.Brush( Gdi.Argb(255, 150, 150, 255) )
		Gdi.OutlineText( 'Total', 2, y+=23, 3)
		Gdi.OutlineText( Pm.TimeStr(totalTime), 2, y += 12, 3)
		Gdi.OutlineText( '$' + totalCost.toFixed(2), 2, y += 12, 3)
	}
//	Gdi.Brush( Gdi.Argb(255, 0, 255, 0) )
//	Gdi.OutlineText( secs2time(totalMon), 2, y += 12, 3)
}
/*
function secs2time( secs)
{
	s = Math.floor(secs % 60)
	m = Math.floor(secs / 60)
	h = Math.floor(m / 60)
	m %= 60
	if(s < 10) s = '0'+s
	if(m < 10) m = '0'+m
	return h + ':' + m + ':' + s
}
*/
// get days of a month for wraparounds
function getDays(month, year)
{
	var ar = new Array( 31, (year % 4 == 0) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 )
	return ar[month]
}

// Load all logs, and reduce if needed
function ReadLogs()
{
	fso = new ActiveXObject( 'Scripting.FileSystemObject' )

	listIn = new Array()
	LoadArray( 'TempIn.log', listIn)
	SaveArray('TempIn.log', listIn)

	listHvac = new Array()
	LoadArray('HVACdata.log', listHvac)
	SaveArray('HVACdata.log', listHvac)

	listStat = new Array()
	LoadArray('stattemp.log', listStat)
	SaveArray('stattemp.log', listStat)

	listOut = new Array()
	LoadArray( 'TempOut.log', listOut)
	SaveArray('TempOut.log', listOut)
	LoadArray('forecast.log', listOut)

	fso = null
}

// Load an array from a comma delimited file
function LoadArray(name, arr)
{
	if(  !fso.FileExists(name) ) return

	tf = fso.OpenTextFile( name, 1, false)

	o = arr.length
	while( !tf.AtEndOfStream)
		arr[o++] = tf.ReadLine().split( ',' )

	tf.Close()
}

// Return start to end of array in hours
function ArrayDuration(arr)
{
	if(arr.length < 2) return 0
	r1 = new Date( arr[0][0] * 1000 )
	r2 = new Date( arr[arr.length-1][0] * 1000 )
	return (r2 - r1) / (1000 * 60*60)		// hours long
}

function arrayMin(arr, n, min)
{
		for ( i = 0; i < arr.length; i++)
		{
			t = +arr[i][n]
			if(min > t) min = t
		}
		return min
}
function arrayMax(arr, n, max)
{
		for ( i = 0; i < arr.length; i++)
		{
			t = +arr[i][n]
			if(max < t) max = t
		}
		return max
}

// Save an array to a file
function SaveArray(name, arr)
{
	max_hours = 24 * 14
	if( ArrayDuration( arr ) <  max_hours) return // check array duration

	tf = fso.CreateTextFile(name, true)

	now = new Date()

	for(i = 0; i < arr.length; i++)
	{
		d = new Date(arr[i][0] * 1000)
		if( ( (now - d) / (1000*60*60) ) <= max_hours) // less than X days old
		{
			line = ''
			for(n = 0; n < arr[i].length; n++)
			{
				line += arr[i][n]
				if(arr[i].length > n+1) line += ','
			}
			tf.WriteLine( line)
		}
	}
	tf.Close()
}
 